# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: protoV1.proto
# Protobuf Python Version: 5.29.3
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    5,
    29,
    3,
    '',
    'protoV1.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rprotoV1.proto\"\xd6\x01\n\x0c\x43ombinedData\x12\x15\n\rabs_timestamp\x18\x01 \x01(\r\x12\x18\n\x07\x64\x65vices\x18\x02 \x03(\x0b\x32\x07.Device\x12\x32\n\x15gateway_points_legacy\x18\x03 \x03(\x0b\x32\x13.GatewayPointLegacy\x12\'\n\x0fposition_points\x18\x04 \x03(\x0b\x32\x0e.PositionPoint\x12\x1f\n\x0bwind_points\x18\x05 \x03(\x0b\x32\n.WindPoint\x12\x17\n\x06uplink\x18\x06 \x01(\x0b\x32\x07.Uplink\"n\n\x06\x44\x65vice\x12\x0c\n\x04type\x18\x01 \x01(\r\x12\x15\n\rserial_number\x18\x02 \x01(\r\x12#\n\rdevice_points\x18\x03 \x03(\x0b\x32\x0c.DevicePoint\x12\x1a\n\x08ui_curve\x18\x04 \x03(\x0b\x32\x08.UiCurve\"\xca\x14\n\x0b\x44\x65vicePoint\x12\x11\n\ttimestamp\x18\x01 \x01(\r\x12\x1a\n\rdevice_status\x18\x02 \x01(\rH\x00\x88\x01\x01\x12\x11\n\x04ldsa\x18\x03 \x01(\rH\x01\x88\x01\x01\x12&\n\x19\x61verage_particle_diameter\x18\x04 \x01(\rH\x02\x88\x01\x01\x12*\n\x1dparticle_number_concentration\x18\x05 \x01(\rH\x03\x88\x01\x01\x12\x18\n\x0btemperature\x18\x06 \x01(\x05H\x04\x88\x01\x01\x12\x1e\n\x11relative_humidity\x18\x07 \x01(\rH\x05\x88\x01\x01\x12\x1c\n\x0f\x62\x61ttery_voltage\x18\x08 \x01(\rH\x06\x88\x01\x01\x12\x1a\n\rparticle_mass\x18\t \x01(\rH\x07\x88\x01\x01\x12\x1b\n\x0e\x63orona_voltage\x18\n \x01(\rH\x08\x88\x01\x01\x12\x1e\n\x11\x64iffusion_current\x18\x0b \x01(\rH\t\x88\x01\x01\x12\x1f\n\x12\x64\x65position_voltage\x18\x0c \x01(\rH\n\x88\x01\x01\x12\x11\n\x04\x66low\x18\r \x01(\rH\x0b\x88\x01\x01\x12\x1d\n\x10\x61mbient_pressure\x18\x0e \x01(\rH\x0c\x88\x01\x01\x12\"\n\x15\x65lectrometer_1_offset\x18\x0f \x01(\x05H\r\x88\x01\x01\x12\"\n\x15\x65lectrometer_2_offset\x18\x10 \x01(\x05H\x0e\x88\x01\x01\x12 \n\x13\x65lectrometer_1_gain\x18\x11 \x01(\rH\x0f\x88\x01\x01\x12 \n\x13\x65lectrometer_2_gain\x18\x12 \x01(\rH\x10\x88\x01\x01\x12%\n\x18\x64iffusion_current_offset\x18\x13 \x01(\x05H\x11\x88\x01\x01\x12!\n\x14particle_number_10nm\x18\x14 \x01(\rH\x12\x88\x01\x01\x12!\n\x14particle_number_16nm\x18\x15 \x01(\rH\x13\x88\x01\x01\x12!\n\x14particle_number_26nm\x18\x16 \x01(\rH\x14\x88\x01\x01\x12!\n\x14particle_number_43nm\x18\x17 \x01(\rH\x15\x88\x01\x01\x12!\n\x14particle_number_70nm\x18\x18 \x01(\rH\x16\x88\x01\x01\x12\"\n\x15particle_number_114nm\x18\x19 \x01(\rH\x17\x88\x01\x01\x12\"\n\x15particle_number_185nm\x18\x1a \x01(\rH\x18\x88\x01\x01\x12\"\n\x15particle_number_300nm\x18\x1b \x01(\rH\x19\x88\x01\x01\x12\x1d\n\x10particle_surface\x18\x1c \x01(\rH\x1a\x88\x01\x01\x12\x1c\n\x0fsigma_size_dist\x18\x1d \x01(\rH\x1b\x88\x01\x01\x12\x1c\n\x0fsteps_inversion\x18\x1e \x01(\rH\x1c\x88\x01\x01\x12\x1b\n\x0e\x63urrent_dist_0\x18\x1f \x01(\rH\x1d\x88\x01\x01\x12\x1b\n\x0e\x63urrent_dist_1\x18  \x01(\rH\x1e\x88\x01\x01\x12\x1b\n\x0e\x63urrent_dist_2\x18! \x01(\rH\x1f\x88\x01\x01\x12\x1b\n\x0e\x63urrent_dist_3\x18\" \x01(\rH \x88\x01\x01\x12\x1b\n\x0e\x63urrent_dist_4\x18# \x01(\rH!\x88\x01\x01\x12\x19\n\x0cpump_current\x18$ \x01(\rH\"\x88\x01\x01\x12\x15\n\x08pump_pwm\x18% \x01(\rH#\x88\x01\x01\x12\x16\n\tcs_status\x18& \x01(\x05H$\x88\x01\x01\x12%\n\x18\x64iffusion_current_stddev\x18\' \x01(\x05H%\x88\x01\x01\x12\'\n\x1a\x64iffusion_current_delay_on\x18( \x01(\rH&\x88\x01\x01\x12(\n\x1b\x64iffusion_current_delay_off\x18) \x01(\rH\'\x88\x01\x01\x12\x19\n\x0cpump_voltage\x18* \x01(\rH(\x88\x01\x01\x12\"\n\x15\x64ifferential_pressure\x18+ \x01(\rH)\x88\x01\x01\x12\x1d\n\x10\x63hannel_pressure\x18, \x01(\rH*\x88\x01\x01\x12\x1e\n\x11supply_voltage_5V\x18- \x01(\rH+\x88\x01\x01\x12!\n\x14positive_voltage_3V3\x18. \x01(\rH,\x88\x01\x01\x12!\n\x14negative_voltage_3V3\x18/ \x01(\rH-\x88\x01\x01\x12\x1b\n\x0eusb_cc_voltage\x18\x30 \x01(\rH.\x88\x01\x01\x12\x1d\n\x10\x66irmware_version\x18\x31 \x01(\rH/\x88\x01\x01\x42\x10\n\x0e_device_statusB\x07\n\x05_ldsaB\x1c\n\x1a_average_particle_diameterB \n\x1e_particle_number_concentrationB\x0e\n\x0c_temperatureB\x14\n\x12_relative_humidityB\x12\n\x10_battery_voltageB\x10\n\x0e_particle_massB\x11\n\x0f_corona_voltageB\x14\n\x12_diffusion_currentB\x15\n\x13_deposition_voltageB\x07\n\x05_flowB\x13\n\x11_ambient_pressureB\x18\n\x16_electrometer_1_offsetB\x18\n\x16_electrometer_2_offsetB\x16\n\x14_electrometer_1_gainB\x16\n\x14_electrometer_2_gainB\x1b\n\x19_diffusion_current_offsetB\x17\n\x15_particle_number_10nmB\x17\n\x15_particle_number_16nmB\x17\n\x15_particle_number_26nmB\x17\n\x15_particle_number_43nmB\x17\n\x15_particle_number_70nmB\x18\n\x16_particle_number_114nmB\x18\n\x16_particle_number_185nmB\x18\n\x16_particle_number_300nmB\x13\n\x11_particle_surfaceB\x12\n\x10_sigma_size_distB\x12\n\x10_steps_inversionB\x11\n\x0f_current_dist_0B\x11\n\x0f_current_dist_1B\x11\n\x0f_current_dist_2B\x11\n\x0f_current_dist_3B\x11\n\x0f_current_dist_4B\x0f\n\r_pump_currentB\x0b\n\t_pump_pwmB\x0c\n\n_cs_statusB\x1b\n\x19_diffusion_current_stddevB\x1d\n\x1b_diffusion_current_delay_onB\x1e\n\x1c_diffusion_current_delay_offB\x0f\n\r_pump_voltageB\x18\n\x16_differential_pressureB\x13\n\x11_channel_pressureB\x14\n\x12_supply_voltage_5VB\x17\n\x15_positive_voltage_3V3B\x17\n\x15_negative_voltage_3V3B\x11\n\x0f_usb_cc_voltageB\x13\n\x11_firmware_version\"@\n\x07UiCurve\x12\x11\n\ttimestamp\x18\x01 \x01(\r\x12\x10\n\x08U_values\x18\x02 \x03(\r\x12\x10\n\x08I_values\x18\x03 \x03(\r\"R\n\x06Uplink\x12\x0c\n\x04type\x18\x01 \x01(\r\x12\x15\n\rserial_number\x18\x02 \x01(\r\x12#\n\ruplink_points\x18\x03 \x03(\x0b\x32\x0c.UplinkPoint\"\x86\t\n\x0bUplinkPoint\x12\x11\n\ttimestamp\x18\x01 \x01(\r\x12\x1d\n\x10\x66irmware_version\x18\x02 \x01(\rH\x00\x88\x01\x01\x12\x1a\n\rdevice_status\x18\x03 \x01(\rH\x01\x88\x01\x01\x12\x1c\n\x0f\x63\x65llular_signal\x18\x04 \x01(\rH\x02\x88\x01\x01\x12 \n\x13\x62\x61ttery_int_voltage\x18\x05 \x01(\rH\x03\x88\x01\x01\x12\x17\n\nvoltage_in\x18\x06 \x01(\rH\x04\x88\x01\x01\x12\x1c\n\x0fusb_voltage_out\x18\x07 \x01(\rH\x05\x88\x01\x01\x12\x1b\n\x0eusb_cc_voltage\x18\x08 \x01(\rH\x06\x88\x01\x01\x12\x1c\n\x0ftemperature_int\x18\t \x01(\x05H\x07\x88\x01\x01\x12\"\n\x15relative_humidity_int\x18\n \x01(\rH\x08\x88\x01\x01\x12\x1e\n\x11temperature_ext_1\x18\x0b \x01(\x05H\t\x88\x01\x01\x12$\n\x17relative_humidity_ext_1\x18\x0c \x01(\rH\n\x88\x01\x01\x12\x1e\n\x11temperature_ext_2\x18\r \x01(\x05H\x0b\x88\x01\x01\x12$\n\x17relative_humidity_ext_2\x18\x0e \x01(\rH\x0c\x88\x01\x01\x12\x15\n\x08sraw_voc\x18\x0f \x01(\rH\r\x88\x01\x01\x12\x16\n\tvoc_index\x18\x10 \x01(\rH\x0e\x88\x01\x01\x12\x1d\n\x10\x61mbient_pressure\x18\x11 \x01(\rH\x0f\x88\x01\x01\x12\x16\n\tco2_ppm_1\x18\x12 \x01(\rH\x10\x88\x01\x01\x12\x16\n\tco2_ppm_2\x18\x13 \x01(\rH\x11\x88\x01\x01\x12\x17\n\npm1_0_mass\x18\x14 \x01(\rH\x12\x88\x01\x01\x12\x17\n\npm2_5_mass\x18\x15 \x01(\rH\x13\x88\x01\x01\x12\x17\n\npm4_0_mass\x18\x16 \x01(\rH\x14\x88\x01\x01\x12\x18\n\x0bpm10_0_mass\x18\x17 \x01(\rH\x15\x88\x01\x01\x12$\n\x17water_level_adc_percent\x18\x18 \x01(\rH\x16\x88\x01\x01\x42\x13\n\x11_firmware_versionB\x10\n\x0e_device_statusB\x12\n\x10_cellular_signalB\x16\n\x14_battery_int_voltageB\r\n\x0b_voltage_inB\x12\n\x10_usb_voltage_outB\x11\n\x0f_usb_cc_voltageB\x12\n\x10_temperature_intB\x18\n\x16_relative_humidity_intB\x14\n\x12_temperature_ext_1B\x1a\n\x18_relative_humidity_ext_1B\x14\n\x12_temperature_ext_2B\x1a\n\x18_relative_humidity_ext_2B\x0b\n\t_sraw_vocB\x0c\n\n_voc_indexB\x13\n\x11_ambient_pressureB\x0c\n\n_co2_ppm_1B\x0c\n\n_co2_ppm_2B\r\n\x0b_pm1_0_massB\r\n\x0b_pm2_5_massB\r\n\x0b_pm4_0_massB\x0e\n\x0c_pm10_0_massB\x1a\n\x18_water_level_adc_percent\"\xba\x02\n\x12GatewayPointLegacy\x12\x16\n\ttimestamp\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x15\n\rserial_number\x18\x02 \x01(\r\x12\x18\n\x10\x66irmware_version\x18\x03 \x01(\r\x12\x13\n\x0b\x66ree_memory\x18\x04 \x01(\r\x12\x11\n\tfree_heap\x18\x05 \x01(\r\x12\x1f\n\x17largest_free_block_heap\x18\x06 \x01(\r\x12\x17\n\x0f\x63\x65llular_signal\x18\x07 \x01(\x05\x12\x17\n\x0f\x62\x61ttery_int_soc\x18\x08 \x01(\r\x12\x17\n\x0f\x62\x61ttery_ext_soc\x18\t \x01(\r\x12\x1b\n\x13\x62\x61ttery_ext_voltage\x18\n \x01(\r\x12\x1c\n\x14\x63harging_ext_voltage\x18\x0b \x01(\rB\x0c\n\n_timestamp\"Z\n\rPositionPoint\x12\x16\n\ttimestamp\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x10\n\x08latitude\x18\x02 \x01(\x02\x12\x11\n\tlongitude\x18\x03 \x01(\x02\x42\x0c\n\n_timestamp\"\x81\x01\n\tWindPoint\x12\x16\n\ttimestamp\x18\x01 \x01(\rH\x00\x88\x01\x01\x12\x17\n\nwind_speed\x18\x02 \x01(\rH\x01\x88\x01\x01\x12\x17\n\nwind_angle\x18\x03 \x01(\rH\x02\x88\x01\x01\x42\x0c\n\n_timestampB\r\n\x0b_wind_speedB\r\n\x0b_wind_angleb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'protoV1_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_COMBINEDDATA']._serialized_start=18
  _globals['_COMBINEDDATA']._serialized_end=232
  _globals['_DEVICE']._serialized_start=234
  _globals['_DEVICE']._serialized_end=344
  _globals['_DEVICEPOINT']._serialized_start=347
  _globals['_DEVICEPOINT']._serialized_end=2981
  _globals['_UICURVE']._serialized_start=2983
  _globals['_UICURVE']._serialized_end=3047
  _globals['_UPLINK']._serialized_start=3049
  _globals['_UPLINK']._serialized_end=3131
  _globals['_UPLINKPOINT']._serialized_start=3134
  _globals['_UPLINKPOINT']._serialized_end=4292
  _globals['_GATEWAYPOINTLEGACY']._serialized_start=4295
  _globals['_GATEWAYPOINTLEGACY']._serialized_end=4609
  _globals['_POSITIONPOINT']._serialized_start=4611
  _globals['_POSITIONPOINT']._serialized_end=4701
  _globals['_WINDPOINT']._serialized_start=4704
  _globals['_WINDPOINT']._serialized_end=4833
# @@protoc_insertion_point(module_scope)
